@@ -22,7 +22,11 @@ class StubGpuPlatformSupportHost : public GpuPlatformSupportHost {
       const base::Callback<void(IPC::Message*)>& send_callback) override {}
 
   void OnChannelDestroyed(int host_id) override {}
-  bool OnMessageReceived(const IPC::Message&) override { return false; }
+  void OnMessageReceived(const IPC::Message&) override {}
+  void OnGpuServiceLaunched(
+      scoped_refptr<base::SingleThreadTaskRunner> ui_runner,
+      scoped_refptr<base::SingleThreadTaskRunner> io_runner,
+      GpuHostBindInterfaceCallback binder) override {}
 };
 
 }  // namespace
