@@ -34,6 +34,7 @@
 #include <QWebEngineDownloadItem>
 #include <QWebEnginePage>
 #include <QWebEngineProfile>
+#include <QWebEngineSettings>
 #include <QWebEngineView>
 #include <httpserver.h>
 
@@ -67,6 +68,9 @@ private Q_SLOTS:
     void downloadViaSetUrl();
     void downloadFileNot1();
     void downloadFileNot2();
+    void downloadDeleted();
+    void downloadDeletedByProfile();
+    void downloadPathValidation();
 
 private:
     void saveLink(QPoint linkPos);
@@ -81,7 +85,8 @@ private:
     QWebEngineProfile *m_profile;
     QWebEnginePage *m_page;
     QWebEngineView *m_view;
-    QSet<QWebEngineDownloadItem *> m_downloads;
+    QSet<QWebEngineDownloadItem *> m_requestedDownloads;
+    QSet<QWebEngineDownloadItem *> m_finishedDownloads;
 };
 
 class ScopedConnection {
@@ -100,13 +105,15 @@ void tst_QWebEngineDownloadItem::initTestCase()
     m_server = new HttpServer();
     m_profile = new QWebEngineProfile;
     m_profile->setHttpCacheType(QWebEngineProfile::NoCache);
+    m_profile->settings()->setAttribute(QWebEngineSettings::AutoLoadIconsForPage, false);
     connect(m_profile, &QWebEngineProfile::downloadRequested, [this](QWebEngineDownloadItem *item) {
-        m_downloads.insert(item);
+        m_requestedDownloads.insert(item);
         connect(item, &QWebEngineDownloadItem::destroyed, [this, item](){
-            m_downloads.remove(item);
+            m_requestedDownloads.remove(item);
+            m_finishedDownloads.remove(item);
         });
         connect(item, &QWebEngineDownloadItem::finished, [this, item](){
-            m_downloads.remove(item);
+            m_finishedDownloads.insert(item);
         });
     });
     m_page = new QWebEnginePage(m_profile);
@@ -123,7 +130,11 @@ void tst_QWebEngineDownloadItem::init()
 
 void tst_QWebEngineDownloadItem::cleanup()
 {
-    QCOMPARE(m_downloads.count(), 0);
+    for (QWebEngineDownloadItem *item : m_finishedDownloads) {
+        item->deleteLater();
+    }
+    QTRY_COMPARE(m_requestedDownloads.count(), 0);
+    QCOMPARE(m_finishedDownloads.count(), 0);
     QVERIFY(m_server->stop());
 }
 
@@ -778,5 +789,183 @@ void tst_QWebEngineDownloadItem::downloadFileNot2()
     QCOMPARE(downloadItem->state(), QWebEngineDownloadItem::DownloadCancelled);
 }
 
+void tst_QWebEngineDownloadItem::downloadDeleted()
+{
+    QPointer<QWebEngineDownloadItem> downloadItem;
+    m_server->setExpectError(true);
+    int downloadCount = 0;
+    int finishedCount = 0;
+    ScopedConnection sc2 = connect(m_profile, &QWebEngineProfile::downloadRequested, [&](QWebEngineDownloadItem *item) {
+        QVERIFY(item);
+        QCOMPARE(item->state(), QWebEngineDownloadItem::DownloadRequested);
+        downloadItem = item;
+        connect(downloadItem, &QWebEngineDownloadItem::finished, [&]() {
+            finishedCount++;
+        });
+        item->accept();
+        downloadCount++;
+    });
+
+    m_page->download(m_server->url(QByteArrayLiteral("/file")));
+    QTRY_COMPARE(downloadCount, 1);
+    QVERIFY(downloadItem);
+    QCOMPARE(finishedCount, 0);
+    downloadItem->deleteLater();
+    QTRY_COMPARE(finishedCount, 1);
+}
+
+void tst_QWebEngineDownloadItem::downloadDeletedByProfile()
+{
+    m_server->setExpectError(true);
+
+    QPointer<QWebEngineProfile> profile(new QWebEngineProfile);
+    profile->setHttpCacheType(QWebEngineProfile::NoCache);
+    profile->settings()->setAttribute(QWebEngineSettings::AutoLoadIconsForPage, false);
+
+    bool downloadFinished = false;
+    QPointer<QWebEngineDownloadItem> downloadItem;
+    connect(profile, &QWebEngineProfile::downloadRequested, [&] (QWebEngineDownloadItem *item) {
+        connect(item, &QWebEngineDownloadItem::finished, [&] () {
+            downloadFinished = true;
+        });
+        downloadItem = item;
+        item->accept();
+    });
+
+    QPointer<QWebEnginePage> page(new QWebEnginePage(profile));
+    page->download(m_server->url(QByteArrayLiteral("/file")));
+
+    QTRY_COMPARE(downloadItem.isNull(), false);
+    QVERIFY(downloadItem);
+
+    page->deleteLater();
+    profile->deleteLater();
+
+    QTRY_COMPARE(downloadFinished, true);
+    QTRY_COMPARE(downloadItem.isNull(), true);
+}
+
+void tst_QWebEngineDownloadItem::downloadPathValidation()
+{
+    const QString fileName = "test.txt";
+    QString downloadPath;
+    QString originalDownloadPath;
+
+    QTemporaryDir tmpDir;
+    QVERIFY(tmpDir.isValid());
+
+    // Set up HTTP server
+    ScopedConnection sc1 = connect(m_server, &HttpServer::newRequest, [&](HttpReqRep *rr) {
+        if (rr->requestMethod() == "GET" && rr->requestPath() == ("/" + fileName)) {
+            rr->setResponseHeader(QByteArrayLiteral("content-type"), QByteArrayLiteral("application/octet-stream"));
+            rr->setResponseHeader(QByteArrayLiteral("content-disposition"), QByteArrayLiteral("attachment"));
+            rr->setResponseBody(QByteArrayLiteral("a"));
+            rr->sendResponse();
+        } else {
+            rr->setResponseStatus(404);
+            rr->sendResponse();
+        }
+    });
+
+    // Set up profile and download handler
+    QPointer<QWebEngineDownloadItem> downloadItem;
+    ScopedConnection sc2 = connect(m_profile, &QWebEngineProfile::downloadRequested, [&](QWebEngineDownloadItem *item) {
+        downloadItem = item;
+        originalDownloadPath = item->path();
+
+        item->setPath(downloadPath);
+        // TODO: Do not cancel download from 5.13. This is for not messing up system download path.
+        // Use m_profile->setDownloadPath(tmpDir.path()) at initialization.
+        if (item->path() != downloadPath)
+            item->cancel();
+        else
+            item->accept();
+
+        connect(item, &QWebEngineDownloadItem::stateChanged, [&, item](QWebEngineDownloadItem::DownloadState downloadState) {
+            if (downloadState == QWebEngineDownloadItem::DownloadInterrupted) {
+                item->cancel();
+            }
+        });
+
+        connect(item, &QWebEngineDownloadItem::finished, [&, item]() {
+            QCOMPARE(item->isFinished(), true);
+            QCOMPARE(item->totalBytes(), item->receivedBytes());
+            QVERIFY(item->receivedBytes() > 0);
+            QCOMPARE(item->page(), m_page);
+        });
+    });
+
+    QString oldPath = QDir::currentPath();
+    QDir::setCurrent(tmpDir.path());
+
+    // Set only the file name.
+    downloadItem.clear();
+    originalDownloadPath = "";
+    downloadPath = fileName;
+    m_page->setUrl(m_server->url("/" + fileName));
+    QTRY_VERIFY(downloadItem);
+    QTRY_COMPARE(downloadItem->state(), QWebEngineDownloadItem::DownloadCompleted);
+    QCOMPARE(downloadItem->interruptReason(), QWebEngineDownloadItem::NoReason);
+    QCOMPARE(downloadItem->path(), fileName);
+
+    // Set only the directory path.
+    downloadItem.clear();
+    originalDownloadPath = "";
+    downloadPath = tmpDir.path();
+    m_page->setUrl(m_server->url("/" + fileName));
+    QTRY_VERIFY(downloadItem);
+    QTRY_COMPARE(downloadItem->state(), QWebEngineDownloadItem::DownloadCancelled);
+    QCOMPARE(downloadItem->interruptReason(), QWebEngineDownloadItem::UserCanceled);
+    QCOMPARE(downloadItem->path(), originalDownloadPath);
+
+    // Set only the directory path with separator.
+    downloadItem.clear();
+    originalDownloadPath = "";
+    downloadPath = tmpDir.path() + QDir::separator();
+    m_page->setUrl(m_server->url("/" + fileName));
+    QTRY_VERIFY(downloadItem);
+    QTRY_COMPARE(downloadItem->state(), QWebEngineDownloadItem::DownloadCancelled);
+    QCOMPARE(downloadItem->interruptReason(), QWebEngineDownloadItem::UserCanceled);
+    QCOMPARE(downloadItem->path(), originalDownloadPath);
+
+    // Set only the directory with the current directory path without ending separator.
+    downloadItem.clear();
+    originalDownloadPath = "";
+    downloadPath = ".";
+    m_page->setUrl(m_server->url("/" + fileName));
+    QTRY_VERIFY(downloadItem);
+    QTRY_COMPARE(downloadItem->state(), QWebEngineDownloadItem::DownloadCancelled);
+    QCOMPARE(downloadItem->interruptReason(), QWebEngineDownloadItem::UserCanceled);
+    QCOMPARE(downloadItem->path(), originalDownloadPath);
+
+    // Set only the directory with the current directory path with ending separator.
+    downloadItem.clear();
+    originalDownloadPath = "";
+    downloadPath = "./";
+    m_page->setUrl(m_server->url("/" + fileName));
+    QTRY_VERIFY(downloadItem);
+    QTRY_COMPARE(downloadItem->state(), QWebEngineDownloadItem::DownloadCancelled);
+    QCOMPARE(downloadItem->interruptReason(), QWebEngineDownloadItem::UserCanceled);
+    QCOMPARE(downloadItem->path(), originalDownloadPath);
+
+
+
+    downloadItem.clear();
+    originalDownloadPath = "";
+    downloadPath = "...";
+    m_page->setUrl(m_server->url("/" + fileName));
+    QTRY_VERIFY(downloadItem);
+    QTRY_COMPARE(downloadItem->state(), QWebEngineDownloadItem::DownloadCancelled);
+#if !defined(Q_OS_WIN)
+    QCOMPARE(downloadItem->interruptReason(), QWebEngineDownloadItem::FileFailed);
+    QCOMPARE(downloadItem->path(), downloadPath);
+#else
+    // Windows interprets the "..." path as a valid path. It will be the current path.
+    QCOMPARE(downloadItem->interruptReason(), QWebEngineDownloadItem::UserCanceled);
+    QCOMPARE(downloadItem->path(), originalDownloadPath);
+#endif // !defined(Q_OS_WIN)
+    QDir::setCurrent(oldPath);
+}
+
 QTEST_MAIN(tst_QWebEngineDownloadItem)
 #include "tst_qwebenginedownloaditem.moc"
